import builtins
import json
from typing import List, Any

from openai.types.beta import Assistant
from openai.types.beta.assistant import Tool


def get_assistant_metadata(assistant:Assistant):
    id:str = assistant.id
    created_at:int = assistant.created_at
    description:str = assistant.description
    file_ids:List[str] = assistant.file_ids
    instructions:str = assistant.instructions
    metadata:builtins.object = assistant.metadata
    model:str = assistant.model
    name:str = assistant.name
    object = assistant.object
    tools:List = assistant.tools

    assistant_metadata = {
        "id": id,
        "name": name,
        "created_at": created_at,
        "description": description,
        "file_ids": file_ids,
        "instructions": instructions,
        "metadata": metadata,
        "model": model,
        "object": object,
        "tools": tools
    }

    # Serialize the dictionary to JSON
    print(assistant_metadata)
    # assistant_metadata_json = json.dumps(assistant_metadata, indent=4)
    # print(assistant_metadata_json)

    return assistant_metadata






from abc import abstractmethod

from openai.types.beta import Thread
from pydantic.v1 import BaseModel


class Assistants(BaseModel):

    @abstractmethod
    def deploy_assistant(self):
        pass

    @abstractmethod
    def register_assistant(self):
        pass

    @abstractmethod
    def init_assistant(self, question="Hi"):
        pass

    @abstractmethod
    def ask_assistant(self,question,thread,assistant_id):
        pass

    @abstractmethod
    def update_assistant(self,assistant_record):
        pass

    @abstractmethod
    def destroy_thread(self):
        pass




import os
import sys

import pymongo
from dotenv import load_dotenv
from pymongo.server_api import ServerApi

from assistants.langregister.base import RegistrationClient

load_dotenv()
MONGO_PASSWORD = os.getenv("MONGO_PASSWORD")


class MongoAtlas(RegistrationClient):
    uri:str=""

    def get_client(self):
        try:
            client = pymongo.MongoClient(self.uri, server_api=ServerApi('1'))
            db = client.assistantstore
            collection = db["assistants"]
            return client, db, collection
            # return a friendly error if a URI error is thrown
        except pymongo.errors.ConfigurationError:
            print("An Invalid URI host error was received. Is your Atlas host name correct in your connection string?")
            sys.exit(1)
        except Exception as e:
            print(e)

    def add_assistant(self, assistant_record):
        client, db, collection = self.get_client()
        try:
            result = collection.insert_many(assistant_record)
            print("mongo result", result)
            return result
        except pymongo.errors.OperationFailure:
            print(
                "An authentication error was received. Are your username and password correct in your connection string?")
            sys.exit(1)

    def get_assistant(self, assistant_record):
        try:
            client, db, collection = self.get_client()
            docs = list(collection.find({"assistant_name": assistant_record["assistant_name"]}))
            assistant_id = ""
            file_id = ""
            if len(docs) > 0:
                for doc in docs:
                    assistant_id = doc['metadata']["id"]
                    file_id = doc["metadata"]["file_ids"]
                return True, assistant_id, file_id
            else:
                return False
        except Exception as e:
            print(e)

    def update_assistant(self, assistant_record):
        try:
            client, db, collection = self.get_client()
            doc = collection.find_one_and_update({"assistant_name": assistant_record["name"]},
                                                 {"$set": {"metadata": assistant_record}}, new=True)
            if doc is not None:
                print("Assistant information updated in the database")
                print(doc)
            else:
                print("No assistant information updates")
            print("\n")
        except Exception as e:
            print(e)

    def delete_assistant(self, assistant_record):
        pass



import os

from dotenv import load_dotenv

from assistants.langregister.mongo_db_atlas import MongoAtlas
from assistants.llm_assistants.openai_assistants import OpenAIRetrievalAssistant

load_dotenv()
OPENAI_API_KEY = os.environ.get('OPENAI_API_KEY')
MONGO_PASSWORD = os.getenv("MONGO_PASSWORD")

instructions = f"""You are a helpful assistant who helps answering questions from a loan estimate"""
file = "../example_data/loan_estimate.pdf"

assistant = OpenAIRetrievalAssistant(instructions=instructions,
                                     assistant_name="loan_estimate_assistant",
                                     file=file
)
mongo_client = MongoAtlas()
mongo_client.uri = "mongodb+srv://rajib76:{MONGO_PASSWORD}@cluster0.cp3rxai.mongodb.net/?retryWrites=true&w=majority".format(
    MONGO_PASSWORD=MONGO_PASSWORD)

# assistant.deploy_assistant(client=mongo_client)

answer, thread, assistant_id = assistant.init_assistant(client=mongo_client)
while True:
    question = input(answer + "\n")
    if question.lower() == "exit":
        answer = "Thanks for using my expertise"
        print(answer)
        deleted = assistant.destroy_thread(thread)
        if deleted:
            print("Thread has been deleted")
            exit(0)
    answer = assistant.ask_assistant(question, thread, assistant_id)



from abc import abstractmethod
from typing import Any

from pydantic.v1 import BaseModel


class RegistrationClient(BaseModel):
    client:Any = None

    @abstractmethod
    def get_client(self):
        pass
    @abstractmethod
    def add_assistant(self,assistant_record):
        pass
    @abstractmethod
    def update_assistant(self,assistant_record):
        pass
    @abstractmethod
    def delete_assistant(self,assistant_record):
        pass


import builtins
from typing import List, Optional

import openai
from openai.types.beta import Assistant, Thread
from openai.types.beta.assistant_create_params import ToolAssistantToolsRetrieval, Tool

from assistants.langregister.mongo_db_atlas import MongoAtlas
from assistants.llm_assistants.assistants import Assistants


class OpenAIRetrievalAssistant(Assistants):
    assistant_id: str = ""
    assistant_name: str = ""
    type: ToolAssistantToolsRetrieval = {"type": "retrieval"}
    created_at: int = 0
    description: str = ""
    file: str = ""
    file_ids: List = []
    instructions: str = ""
    metadata: dict = {}
    model: str = "gpt-4-1106-preview"
    object: str = "assistant"
    tools: List[Tool] = None

    def deploy_assistant(self, client=MongoAtlas()):
        try:
            self.register_assistant(client)
            file = openai.files.create(
                file=open(self.file, "rb"),
                purpose='assistants'
            )
            assistant = openai.beta.assistants.create(
                name=self.assistant_name,
                instructions=self.instructions,
                model=self.model,
                tools=[self.type],
                file_ids=[file.id]
            )
            # assistant_record = {"assistant_name": assistant.name, "assistant_id": assistant.id, "file_id": file.id}
            assistant_record = self.get_assistant_metadata(assistant)
            self.update_assistant(assistant_record, client)
            return assistant
        except Exception as e:
            print(e)

    def register_assistant(self, client=MongoAtlas()):
        duplicate_assistant = False
        assistant_record = {"assistant_name": self.assistant_name}
        try:
            duplicate_assistant, _, _ = client.get_assistant(assistant_record)
        except Exception as e:
            print(e)

        if duplicate_assistant:
            raise Exception("Assistant with same name exists")
        else:
            client.add_assistant([assistant_record])

    def update_assistant(self, assistant_record, client=MongoAtlas()):
        client.update_assistant(assistant_record)

    def get_answer(self, run, thread, type: [Optional]):
        if type == "init":
            print("Initializing the assistant")
        else:
            print("Looking for an answer to your question...")

        while run.status != "completed":
            run = openai.beta.threads.runs.retrieve(
                thread_id=thread.id,
                run_id=run.id
            )

        if type == "init":
            print("Done initializing")
        else:
            print("Phew, that was a lot of reading...")

        messages = openai.beta.threads.messages.list(
            thread_id=thread.id
        )

        annotations = messages.data[0].content[0].text.annotations
        message_content = messages.data[0].content[0].text.value
        return annotations, message_content

    def init_assistant(self, question="Hi Summarizer", client=MongoAtlas()):
        assistant_record = {"assistant_name": self.assistant_name}
        _, assistant_id, file_id = client.get_assistant(assistant_record)
        thread = openai.beta.threads.create(
            messages=[
                {
                    "role": "user",
                    "content": question,
                    "file_ids": file_id
                }
            ]
        )

        run = openai.beta.threads.runs.create(
            thread_id=thread.id,
            assistant_id=assistant_id
        )

        _, answer = self.get_answer(run, thread, type="init")
        return answer, thread, assistant_id

    def ask_assistant(self, question, thread, assistant_id):
        messages = openai.beta.threads.messages.create(
            thread_id=thread.id,
            content=question,
            role="user"
        )
        run = openai.beta.threads.runs.create(
            thread_id=thread.id,
            assistant_id=assistant_id)
        _, answer = self.get_answer(run, thread, type="")
        answer = answer + "\n\n" + "What else can I help with?"
        return answer

    def destroy_thread(self, thread: Thread = None):
        try:
            if thread is None:
                raise Exception("Thread was not deleted as no thread id passed")
            status = openai.beta.threads.delete(thread_id=thread.id)
            if status.deleted:
                print("Deleting thread id: ", status.id)
                return True
            else:
                print(status)
                raise Exception("Thread could not be deleted")
        except Exception as e:
            print(e)

    def get_assistant_metadata(self, assistant: Assistant):
        id: str = assistant.id
        created_at: int = assistant.created_at
        description: str = assistant.description
        file_ids: List[str] = assistant.file_ids
        instructions: str = assistant.instructions
        metadata: builtins.object = assistant.metadata
        model: str = assistant.model
        name: str = assistant.name
        object = assistant.object
        tools: List = assistant.tools

        assistant_metadata = {
            "id": id,
            "name": name,
            "created_at": created_at,
            "description": description,
            "file_ids": file_ids,
            "instructions": instructions,
            "metadata": metadata,
            "model": model,
            "object": object,
            "tools": str(tools)
        }

        return assistant_metadata



import os

from dotenv import load_dotenv

from assistants.langregister.mongo_db_atlas import MongoAtlas
from assistants.llm_assistants.openai_assistants import OpenAIRetrievalAssistant

load_dotenv()
OPENAI_API_KEY = os.environ.get('OPENAI_API_KEY')
MONGO_PASSWORD = os.getenv("MONGO_PASSWORD")


class SummarizationAgent():
    def __init__(self):
        self.instructions = f"""You are a helpful summarization agent.You help summarize the content provided to you"""
        self.file = "/Users/joyeed/assistants/assistants/assistants/example_data/gen_ai.pdf"
        self.assistant = OpenAIRetrievalAssistant(instructions=self.instructions,
                                                  assistant_name="summarize_assistant",
                                                  file=self.file)
        self.mongo_client = MongoAtlas()
        self.mongo_client.uri = "mongodb+srv://rajib76:{MONGO_PASSWORD}@cluster0.cp3rxai.mongodb.net/?retryWrites=true&w=majority".format(
            MONGO_PASSWORD=MONGO_PASSWORD)

    def deploy_assistant(self):
        self.assistant.deploy_assistant(self.mongo_client)

    def ask_assisant(self):
        answer, thread, assistant_id = self.assistant.init_assistant(client=self.mongo_client)
        while True:
            question = input(answer + "\n")
            if question.lower() == "exit":
                answer = "Thanks for using my expertise"
                print(answer)
                deleted = self.assistant.destroy_thread(thread)
                if deleted:
                    print("Thread has been deleted")
                    exit(0)
            answer = self.assistant.ask_assistant(question, thread, assistant_id)
            print(answer)


if __name__ == "__main__":
    sc = SummarizationAgent()
    sc.ask_assisant()
